// mini timer MaE

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>

//************* the only user-definable varibles *****************************

#define VALMAX	3600	//max value in seconds, max 65476
#define	TRESH	59	//treshold between minutes and seconds visualization, please note offset will remain

//****************************************************************************


#define PENC	PINB
#define ENCA	PB3
#define ENCB	PB4
#define PBTN	PINB
#define BTN	PB2
#define PBTNO	PORTB
#define DDRBTN	DDRB
#define POUT	PORTB
#define OUT	PB1

#define SCALE	122	//means 1Hz


uint8_t Flags;
uint16_t Timer;
uint16_t Set;
enum {TICK, RUN , BUTTON, BLINK, BEEP,ALLOW};

uint8_t Scaler;

void load(void) {
//load 'Set' from eeprom
   if (eeprom_read_byte(0)==1) {
        Set = eeprom_read_word(2);
   } else {
        Set=10;
   }
   Timer=Set;
}

void save(void) {
//save 'Set' to eeprom
   eeprom_write_byte(0,1);
   eeprom_write_word(2, Set );

}



int bcd(int value) {
    switch (value) {
	case 0: return 0b01101111;
	case 1: return 0b00000110;
	case 2: return 0b01110011;
	case 3: return 0b01010111;
	case 4: return 0b00011110;
	case 5: return 0b01011101;
	case 6: return 0b01111101;
	case 7: return 0b00000111;
	case 8: return 0b01111111;
	case 9: return 0b01011111;
    }
}


void update(void) {
// shows timer value, in seconds if less than treshold, in minutes if greater than treshold
// dot point is off for seconds, on still or blinking for minutes
uint8_t fig0, fig1;
if (Timer>TRESH){	//minutes, dot point blinking (if running) or still (if stopped)
   if (Flags&(1<<RUN)){
	if (Flags&(1<<BLINK)){
		Flags&=~(1<<BLINK);
		PORTB|=(1<<PB0);
	} else {
		Flags|=(1<<BLINK);
		PORTB&=~(1<<PB0);
	}
   } else {
	PORTB&=~(1<<PB0);
   }
    fig1=bcd((uint8_t)(Timer/600));
    fig0=bcd(((uint8_t)(Timer/60))%10);
} else {	//seconds, no dot point
   PORTB|=(1<<PB0);
   fig1=bcd((uint8_t)(Timer/10));
   fig0=bcd(Timer%10);
}//endif sec/min
PORTD=~((fig0&127)|((fig1&64)<<1));
PORTC=~(fig1&63);

if (Flags&(1<<BEEP)) {
	Flags |= (1<<BEEP);
	DDRBTN |= (1<<BTN);
	PBTNO &= ~(1<<BTN);
} else {
	PBTNO|=(1<<BTN);
	Flags&=~(1<<BEEP);
	DDRBTN &= ~(1<<BTN);
}


}


void main(void) {

    wdt_disable();

    DDRB =  0b00000011;
    DDRC =  0b01111111;
    DDRD =  0b11111111;
    PORTB = 0b00111101;
    PORTC = 0b00111111;
    PORTD = 0b11111111;

    TCCR0 = _BV(CS02); // clk/256 ... 8bit timer .... 122Hz result
    TIMSK |= _BV(TOIE0);

	Scaler=SCALE;

	Flags=0;

	load();

	update();

	sei();

    uint8_t encAp = (PENC & (1<<ENCA) );

    while(1) {
        while(((PENC & (1<<ENCA) ) == encAp )|| (Flags&(1<<RUN))) {  //wait for A change, but only if not running!

		if (Flags&(1<<TICK)) {	// 1 Hz
		    Flags&=~(1<<TICK);
		    if (Flags&(1<<BUTTON)) {
			Flags&=~(1<<BUTTON);
			Flags&=~(1<<ALLOW);
		    }else{
			Flags|=(1<<ALLOW);
		    }
		    if (Flags&(1<<RUN)) {
			if (Timer==0) {	//stop
				Flags&=~(1<<RUN);
				Flags&=~(1<<BEEP);
				POUT&=~(1<<OUT);
				Flags&=~(1<<ALLOW);
				Timer=Set;
			}else{	//decrement
				Timer--;
				if (Timer==0) Flags|=(1<<BEEP);
			}
			update();
		    } //end run
		} //end 1Hz

		if ( (!(PBTN & (1<<BTN))) && (!(Flags&(1<<BEEP))) ) 	//button pressed
		{
		    Flags|=(1<<BUTTON);
		    if (Flags&(1<<ALLOW)) {
			Flags&=~(1<<ALLOW);
			if (Flags&(1<<RUN))  {
				Flags &= ~(1<<RUN);
				POUT &= ~(1<<OUT);
				update();
			} else {
				Flags |= (1<<RUN);
				POUT |= (1<<OUT);
				Set=Timer;
				save();
			}
		     }//end allowed
                  }//end button pressed
	}//end while A==A
        //encoder moved
        uint8_t enc= PENC & ((1<<ENCA)|(1<<ENCB));   //mask
        encAp=enc&(1<<ENCA);
        if ((enc==0) || (enc==((1<<ENCA)|(1<<ENCB)))) {      //B==A
            if (Timer>0) {
		if (Timer>(TRESH+1)) Timer-=60; else Timer--;	//decrement 1 min if minutes, 1 sec if seconds
	    }
        }
        else if ((enc==(1<<ENCB))||(enc==(1<<ENCA))) {    //B!=A
            if (Timer<VALMAX) {
		if (Timer>TRESH) Timer+=60; else Timer++;	//increment
	    }
        }
	update();
    }	//end while(1)

}


ISR (TIMER0_OVF_vect){
  if (--Scaler==0) {
    Scaler=SCALE;
    //this code is executed at 1Hz rate.

	Flags|=(1<<TICK);

    }
  }
